<?php

class Page extends model {

    const DB_TABLE = 'pages';

    public int $id;
    public string $title;
    public string $md;
    public string $html;
    public int $ts;
    public int $updateTs;
    public bool $visible;
    public bool $renderTitle;
    public string $shortName;

    function edit(array $fields) {
        $fields['update_ts'] = time();
        if ($fields['md'] != $this->md || $fields['render_title'] != $this->renderTitle) {
            $md = $fields['md'];
            if ($fields['render_title'])
                $md = '# '.$fields['title']."\n\n".$md;
            $fields['html'] = markup::markdownToHtml($md);
        }
        parent::edit($fields);
    }

    function isUpdated(): bool {
        return $this->updateTs && $this->updateTs != $this->ts;
    }

    function getHtml(bool $is_retina, string $user_theme): string {
        $html = $this->html;
        $html = markup::htmlImagesFix($html, $is_retina, $user_theme);
        return $html;
    }

    function getUrl(): string {
        return "/{$this->shortName}/";
    }

    function updateHtml(): void {
        $html = markup::markdownToHtml($this->md);
        $this->html = $html;
        DB()->query("UPDATE pages SET html=? WHERE short_name=?", $html, $this->shortName);
    }

}

class pages {

    static function add(array $data): bool {
        $db = DB();
        $data['ts'] = time();
        $data['html'] = markup::markdownToHtml($data['md']);
        if (!$db->insert('pages', $data))
            return false;
        return true;
    }

    static function delete(Page $page): void {
        DB()->query("DELETE FROM pages WHERE short_name=?", $page->shortName);
        previous_texts::delete(PreviousText::TYPE_PAGE, $page->get_id());
    }

    static function getByName(string $short_name): ?Page {
        $db = DB();
        $q = $db->query("SELECT * FROM pages WHERE short_name=?", $short_name);
        return $db->numRows($q) ? new Page($db->fetch($q)) : null;
    }

    /**
     * @return Page[]
     */
    static function getAll(): array {
        $db = DB();
        return array_map('Page::create_instance', $db->fetchAll($db->query("SELECT * FROM pages")));
    }

}